# A script for Metropolis-Hastings (Ch.9) for Bayesian Method (2nd edition) by Gill
# Written by Kuniaki Nemoto


# A function to produce bivariate normal density values as shown on p.393.
# Note "dmultinorm" is already in the BaM package;
# if you want to use the function below,
# make sure you change "dmultinorm" to "dmultinorm.Gill". It works.
dmultinorm.Gill <- function(xval, yval, mu.vector, sigma.matrix) {
  normalizer <- (2*pi*sigma.matrix[1,1]*sigma.matrix[2,2]
                 *sqrt(1-sigma.matrix[1,2]^2))^(-1)
  like <- exp(-(1/(2*(1-sigma.matrix[1,2]^2)))*(
    ((xval-mu.vector[1])/sigma.matrix[1,1])^2
    -2*sigma.matrix[1,2]*(((xval-mu.vector[1])
                           /sigma.matrix[1,1])*
                            ((yval-mu.vector[2])/sigma.matrix[2,2]))
    +((yval-mu.vector[2])/sigma.matrix[2,2])^2))
  normalizer*like
  
}


# The Metropolis-Hastings algorithm script on p.375.
metropolis <- function(theta.matrix,reps,I.mat)  {
  for (i in 2:reps)  {
    theta.star <- mvrnorm(1,theta.matrix[(i-1),],I.mat)/(sqrt(rchisq(2,5)/5))
    a <- dmultinorm(theta.star[1],theta.star[2],c(0,0),I.mat)/ 
         dmultinorm(theta.matrix[(i-1),1],theta.matrix[(i-1),2],c(0,0),I.mat) 
    if (a > runif(1)) theta.matrix[i,] <- theta.star
    else theta.matrix[i,] <- theta.matrix[(i-1),]
  }
  theta.matrix
}


# Run the models w/ some initial values and seed.
set.seed(20131121)
num.sims <- 1000 # Run it 1000 times
initial.values <- matrix(c(0,0),num.sims,2)
covariance.matrix <- matrix(c(1,-0.9,-0.9,1),2,2) # The covariance matrix as shown in p.374
metropolis.walk <- metropolis(theta.matrix=initial.values,reps=num.sims,I.mat=covariance.matrix) 


# The script on pp.392-3 to show Figure 9.4 on p.375.
plot.walk.MH <- function(walk.mat) {
  plot(walk.mat[1,1], walk.mat[1,2], type="n",
    xlim=round(range(walk.mat[,1])*1.2),
    ylim=round(range(walk.mat[,2])*1.2),
    xlab="", ylab="")
  for(i in 1:(nrow(walk.mat)-1)) {
    segments(walk.mat[i,1],walk.mat[i,2],
             walk.mat[(i+1),1],walk.mat[(i+1),2])
  }
}

par(mfrow=c(1,2), mar=c(2,2,2,2), oma=c(1,1,3,1))
plot(metropolis.walk[801:1000,], pch=".", xlab="", ylab="", # "theta.matrix" is a local; changed it to "metropolis.walk"
     xlim=c(-3,3), ylim=c(-3,3), cex=3)
plot.walk.MH(metropolis.walk[801:1000,]) # "theta.matrix" is a local; changed it to "metropolis.walk"
mtext(outer=TRUE, side=3, cex=1.2, "Metropolis-Hastings Demonstration, Bivariate Normal")


# Hit-and-Run algorithm as shown on p.377.
hit.run <- function(theta.mat, reps, I.mat) {
  for (i in 2:reps) {
    u.vec <- c(runif(1,0,pi/2), runif(1,pi/2,pi),
               runif(1,pi,3*pi/2),
               runif(1,3*pi/2,2*pi))
    u.dr <- sample(u.vec, size=1, prob=c(1/3, 1/6, 1/3, 1/6))
    g.ds <- rgamma(1,1,1)
    xy.theta <- c(g.ds*cos(u.dr), g.ds*sin(u.dr)) + theta.mat[(i-1),]
    a <- dmultinorm(xy.theta[1], xy.theta[2], c(0,0), I.mat)
    r.uniform <- runif(1)
    if (a > r.uniform) theta.mat[i,] <- xy.theta
    else theta.mat[i,] <- theta.mat[(i-1),]
  }
  theta.mat
}


# The script on p.393 to show the graph on p.378.
num.sims <- 10000
Sig.mat <- matrix(c(1.0, 0.95, 0.95, 1.0), 2, 2)
walks <- rbind(c(-3,-3), matrix(NA, nrow=(num.sims - 1), ncol=2))
walks <- hit.run(walks, num.sims, Sig.mat)
z.grid <- outer(seq(-3, 3, length=100), seq(-3, 3, length=100),
                FUN=dmultinorm, c(0,0), Sig.mat)
contour(seq(-3, 3, length=100), seq(-3, 3, length=100), z.grid,
        levels=c(0.05, 0.1, 0.2))
points(walks[5001:num.sims,],pch=".")


